@@ -245,8 +245,12 @@ mod std_support {
 /// 
 /// ### Sin
 /// Take the elemtent-wise sine using a Taylor approximation of sine x.
+/// 
+/// Sin is calculated using the following polynomial:
+/// `sinx = x -( x^3/6.0 )+( x^5/120.0 )-( x^7/5040.0 )+( x^9/362880.0 )`
+/// 
 /// Self must be wrapped to the -π=<x<π range.
-/// * `pi` The integer level which represents π in the input data.
+/// * `pi`   The integer level which represents π in the input data.
 /// * `norm` The integer level which represents 1 in the output data.
 /// ```rust
 /// use integer_array as ia;
@@ -254,21 +258,66 @@ mod std_support {
 /// ia::declare_array_real!( Arr8, 8);
 /// let mut x = Arr8::ramp(0,60);
 /// x = x.wrap_phase( 180 );
-///    assert_eq!{x.sin( 180, 100).data, [0, 86, 86, 0, -86, -86, 0, 86] };
+/// assert_eq!{x.sin( 180, 100).data, [0, 86, 86, 0, -86, -86, 0, 86] };
 /// ```
 /// 
+/// Below is the the taylor approximation for sine compared to the Julia native sin function.
+/// The below plot is generated with double presicion floationg point for demonstrative purposes.
+/// In the figure it is apparent that there is greater error in the Taylor approximation further from origo.
+/// Therefore, 
+/// ![Image](../../../numerical_verificatons/figures/sin/time_domain_sinx.png?raw=true)
+/// To counter these, the fact that all quarters of the sine(x) function are mirrored versions of each other. 
+/// Therefore the first quarters, having the least error, which can be seen in the time domain plot above, are used for all values of x.
+/// Resulting in a practically ideal sine function, as can be seen in the frequency domain comparison below. 
+/// ![Image](../../../numerical_verificatons/figures/sin/time_domain_sinx.png?raw=true)
+/// 
+/// ### Cos
+/// Take the elemtent-wise cosine using a Taylor approximation of cos x.
+/// 
+/// Cos is calculated using the following polynomial:
+/// `cosx = 1 -( x^2/2 )+( x^4/24.0 )-( x^6/720.0 )+( x^8/40320.0 )`
+/// 
+/// Self must be wrapped to the -π=<x<π range.
+/// * `pi`   The integer level which represents π in the input data.
+/// * `norm` The integer level which represents 1 in the output data.
+/// ```rust
+/// use integer_array as ia;
+/// use ia::trait_definitions::*;
+/// ia::declare_array_real!( Arr8, 8);
+/// let mut x = Arr8::ramp(0,60);
+/// x = x.wrap_phase( 180 );
+/// assert_eq!{x.cos( 180, 100).data, [100, 50, -50, -100, -50, 50, 100, 50] };
+/// ```
+/// 
+/// A first-quarter method as described for the sine implementation is also used on cosine. The pure Taylor approximation is displayed below. 
+/// ![Image](../../../numerical_verificatons/figures/cos/time_domain.png?raw=true)
+/// With the first-quarter method, the resulting cosine power spectrum is displayed below.
+/// ![Image](../../../numerical_verificatons/figures/com/frequency_domain2.png?raw=true)
+/// 
+/// 
 /// ### Tan
 /// Take the element-wise tan using a Taylor approximation of tan x.
-/// * 'pi' The integer level which represents π in the input data.
-/// * 'norm' The integer level which represents 1 in the output data.
+/// 
+/// Tan is calculated using the following polynomial:
+/// tanx = x+( x^3/3 )+( x^5*2/15.0 )+( x^7*17/315.0 )+( x^9*62/2835.0 )+( x^11*1382/155925.0 )+( x^13*21844/6081075.0 )+( x^15*929569/638512875.0 );
+
+/// Self must be wrapped to the -π/2=<x<π/2 range.
+/// The function is based on a Taylor expansion. Its error increases as |x| approaches π/2.
+/// 
+/// * `pi`   The integer level which represents π in the input data.
+/// * `norm` The integer level which represents 1 in the output data.
 /// ```rust
 /// use integer_array as ia;
 /// use ia::trait_definitions::*;
 /// ia::declare_array_real!( Arr8, 8);
 /// let x = Arr8::ramp(0,20);
-/// assert_eq!{x.tan( 180, 100).data, [0, 36, 83, 155, 289, 630, 1794, 5875] };
+/// assert_eq!{x.tan( 180, 100).data, [0, 36, 83, 158, 373, 2155, 19696, 158268] };
 /// ```
 /// 
+/// Below is the the taylor approximation for tan compared to the Julia native tan function.
+/// The below plot is generated with double presicion floationg point for demonstrative purposes.
+/// ![Image](../../../numerical_verificatons/figures/tan/time_domain.png?raw=true) 
+/// 
 /// ## Estimator utilites:
 /// ### Max and min:
 /// The maimum and minimum value in the array can be found through the `.max()` and `.min()` traits respectively.
@@ -410,9 +459,9 @@ macro_rules! declare_array_real{
             /// Return the elemtent-wise square root using the 
             /// Babylonian square root implementation.
             fn sqrt( &self ) -> Self {
-                let mut temp = self.data.clone();
+                let mut r_array = self.clone();
                 for index in 0..$N {
-                    let item = self.data[index];
+                    let item = self[index];
                     // Initial approximation
                     let mut root:i32 = item/2;
                     let mut y:i32 = 1;
@@ -423,11 +472,9 @@ macro_rules! declare_array_real{
                         root = (root + y) / 2;
                         y = item / root;
                     }
-                    temp[index] = root;
+                    r_array[index] = root;
                 } 
-                Self {
-                    data: temp
-                }
+                return r_array;
             }
         }
 
@@ -686,7 +733,7 @@ macro_rules! declare_array_real{
         }
 
         impl integer_array::trait_definitions::TrigonometryTraits for $name {
-            /// Take the elemtent-wise sine using a Taylor approximation of sine x.
+            /// Take the elemtent-wise sine using a Taylor approximation of sin(x).
             /// Self must be wrapped to the -π=<x<π range.
             /// * `pi` The integer level which represents π in the input data.
             /// * `norm` The integer level which represents 1 in the output data.
@@ -700,8 +747,8 @@ macro_rules! declare_array_real{
 
                 for idx in 0..$N {
                     let mut x:f32 = (self[idx] as f32)/(norm_pi as f32 ) *cnst::PI;
-                    // Ensure that the angle is within the accurate range of the tailor series. 
 
+                    // Ensure that the angle is within the accurate range of the tailor series. 
                     if x < -PI_HALF
                     {   
                         let delta = x+PI_HALF;
@@ -715,27 +762,67 @@ macro_rules! declare_array_real{
 
                     // Calculate sine by using 
                     let sinx:f32 = x-( util::fpowi(x,3)/6.0 )+( util::fpowi(x,5)/120.0 )-( util::fpowi(x,7)/5040.0 )+( util::fpowi(x,9)/362880.0 );
-                    assert!( sinx.abs()<1.0, "sinx = {}, x = {}", sinx, x);
 
                     r_array[idx] = ( sinx*(norm as f32) ) as i32;
                 } 
                 return r_array;
             }
-            /// Take the element-wise tan using a Taylor approximation of tan x.
+
+            /// Take the elemtent-wise cosine using a Taylor approximation of cos(x).
             /// Self must be wrapped to the -π=<x<π range.
             /// * `pi` The integer level which represents π in the input data.
             /// * `norm` The integer level which represents 1 in the output data.
+            fn cos( &self, norm_pi:i32, norm:i32 ) -> Self {
+                use integer_array::utility_functions as util;
+                use integer_array::constants as cnst;
+
+                const PI_HALF:f32 = cnst::PI/2.0;
+
+                let mut r_array = $name::zeros();
+                for idx in 0..$N {
+                    // Add a π/2 phase shift and run the sine computation from above.
+                    let mut x:f32 = ((self[idx] as f32)/(norm_pi as f32 ) *cnst::PI);
+                    
+                    let cosx:f32;
+                    // Ensure that the angle is within the accurate range of the tailor series. 
+                    if x < -PI_HALF
+                    {   
+                        let delta = x+PI_HALF;
+                        x = -PI_HALF+delta.abs();
+                        cosx = -(1.0-( util::fpowi(x,2)/2.0 )+( util::fpowi(x,4)/24.0 )-( util::fpowi(x,6)/720.0 )+( util::fpowi(x,8)/40320.0 ));
+
+                    }
+                    else if PI_HALF < x
+                    {
+                        let delta = x-PI_HALF;
+                        x = PI_HALF-delta.abs();
+                        cosx = -(1.0-( util::fpowi(x,2)/2.0 )+( util::fpowi(x,4)/24.0 )-( util::fpowi(x,6)/720.0 )+( util::fpowi(x,8)/40320.0 ));
+                    }
+                    else
+                    {
+                        cosx = 1.0-( util::fpowi(x,2)/2.0 )+( util::fpowi(x,4)/24.0 )-( util::fpowi(x,6)/720.0 )+( util::fpowi(x,8)/40320.0 );
+                    }
+                    r_array[idx] = ( cosx*(norm as f32) ) as i32;
+                } 
+                return r_array;
+            }
+
+            /// Take the element-wise tan using a Taylor approximation of tan x.
+            /// Self must be wrapped to the -π/2=<x<π/2 range.
+            /// The function is based on a Taylor expansion. Its error increases as |x| approaches π/2.
+            /// * `pi` The integer level which represents π in the input data.
+            /// * `norm` The integer level which represents 1 in the output data.
             fn tan( &self, norm_pi:i32, norm:i32 ) -> Self {
                 let mut temp = self.data.clone();
                 
                 use integer_array::utility_functions as util;
                 use integer_array::constants as cnst;
 
-
                 for idx in 0..$N {
                     let x:f32 = (temp[idx] as f32)*cnst::PI/(norm_pi as f32 );
                     // Calculate tan by using a polynomial 
-                    let tanx:f32 = x+(util::fpowi(x,3)/3.0 )+( util::fpowi(x,5)*2.0/15.0 )-( util::fpowi(x,7)*17.0/315.0 )+( util::fpowi(x,9)*62.0/2835.0 );
+                    let tanx:f32 = x+( util::fpowi(x,3)/3.0 )+( util::fpowi(x,5)*2.0/15.0 )-( util::fpowi(x,7)*17.0/315.0 )+( util::fpowi(x,9)*62.0/2835.0 )+( util::fpowi(x,11)*1382.0/155925.0 )
+                                    +( util::fpowi(x,13)*21844.0/6081075.0 )+( util::fpowi(x,15)*929569.0/638512875.0 );
                     temp[idx] = ( tanx*(norm as f32) ) as i32;
                 } 
                 Self {
